from typing import Iterable

# isort: split
from ansys.meshing.prime.autogen.surfer import Surfer as _Surfer

# isort: split
from ansys.meshing.prime.autogen.coreobject import CoreObject
from ansys.meshing.prime.autogen.surferstructs import (
    LocalSurferParams,
    LocalSurferResults,
    SurferParams,
    SurferResults,
)
from ansys.meshing.prime.core.model import Model


class Surfer(CoreObject):
    __doc__ = _Surfer.__doc__

    def __init__(self, model: Model):
        """__init__(Surfer self, Model model)"""
        self._model = model

    def mesh_topo_faces(
        self, part_id: int, topo_faces: Iterable[int], params: SurferParams
    ) -> SurferResults:
        """Performs meshing on the given topofaces with provided parameters.

        Parameters
        ----------
        part_id : int
            Id of part.
        topo_faces : Iterable[int]
            Ids of topofaces.
        params : SurferParams
            Surfer Parameters.

        Returns
        -------
        SurferResults
            Returns the SurferResults.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> surfer_params = prime.SurferParams(model)
        >>> results = surfer.mesh_topo_faces(part_id, topo_faces, surfer_params)

        """
        with _Surfer(model=self._model, part_id=part_id) as surfer:
            return surfer.mesh_topo_faces(topo_faces, params)

    def remesh_face_zonelets_locally(
        self,
        part_id: int,
        face_zonelets: Iterable[int],
        register_id: int,
        local_surfer_params: LocalSurferParams,
    ) -> LocalSurferResults:
        """Remesh the given face zonelets locally at the registered faces with provided parameters.

        Parameters
        ----------
        part_id : int
            Id of part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Register id of the target faces.
        local_surfer_params : LocalSurferParams
            Local surfer Parameters.

        Returns
        -------
        LocalSurferResults
            Returns the LocalSurferResults.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> local_surfer_params = prime.LocalSurferParams(model)
        >>> results = surfer.remesh_face_zonelets_locally(part_id,
                        face_zonelets, register_id, local_surfer_params)

        """
        with _Surfer(model=self._model, part_id=part_id) as surfer:
            return surfer.remesh_face_zonelets_locally(
                face_zonelets, register_id, local_surfer_params
            )

    def remesh_face_zonelets(
        self,
        part_id: int,
        face_zonelets: Iterable[int],
        edge_zonelets: Iterable[int],
        params: SurferParams,
    ) -> SurferResults:
        """Performs meshing on the given face zonelets with provided parameters.

        Parameters
        ----------
        part_id : int
            Id of part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        edge_zonelets : Iterable[int]
            Ids of edge zonelets.
        params : SurferParams
            Surfer parameters.

        Returns
        -------
        SurferResults
            Returns the SurferResults.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> surfer_params = prime.SurferParams(model)
        >>> results = surfer.remesh_face_zonelets(part_id,
                        face_zonelets, edge_zonelets, surfer_params)

        """
        with _Surfer(model=self._model, part_id=part_id) as surfer:
            return surfer.remesh_face_zonelets(face_zonelets, edge_zonelets, params)

    def initialize_surfer_params_for_wrapper(self) -> SurferParams:
        """Initialize surfer parameters to mesh surfaces generated by wrapper.


        Returns
        -------
        SurferParams
            Returns the SurferParams initialized for wrapper inputs.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> surfer_params = surfer.initialize_surfer_params_for_wrapper()

        """
        with _Surfer(model=self._model, part_id=-1) as surfer:
            return surfer.initialize_surfer_params_for_wrapper()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        pass



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class VolumeControlParams(CoreObject):
    """Volume control parameters are used to define the volume type in the volume control.
    """
    _default_params = {}

    def __initialize(
            self,
            cell_zonelet_type: CellZoneletType):
        self._cell_zonelet_type = CellZoneletType(cell_zonelet_type)

    def __init__(
            self,
            model: CommunicationManager=None,
            cell_zonelet_type: CellZoneletType = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumeControlParams.

        Parameters
        ----------
        model: Model
            Model to create a VolumeControlParams object with default parameters.
        cell_zonelet_type: CellZoneletType, optional
            Cell zonelet type is used to define the type of the associated volume.
        json_data: dict, optional
            JSON dictionary to create a VolumeControlParams object with provided parameters.

        Examples
        --------
        >>> volume_control_params = prime.VolumeControlParams(model = model)
        """
        if json_data:
            self.__initialize(
                CellZoneletType(json_data["cellZoneletType"] if "cellZoneletType" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [cell_zonelet_type])
            if all_field_specified:
                self.__initialize(
                    cell_zonelet_type)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumeControlParams")
                    json_data = param_json["VolumeControlParams"] if "VolumeControlParams" in param_json else {}
                    self.__initialize(
                        cell_zonelet_type if cell_zonelet_type is not None else ( VolumeControlParams._default_params["cell_zonelet_type"] if "cell_zonelet_type" in VolumeControlParams._default_params else CellZoneletType(json_data["cellZoneletType"] if "cellZoneletType" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            cell_zonelet_type: CellZoneletType = None):
        """Set the default values of VolumeControlParams.

        Parameters
        ----------
        cell_zonelet_type: CellZoneletType, optional
            Cell zonelet type is used to define the type of the associated volume.
        """
        args = locals()
        [VolumeControlParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumeControlParams.

        Examples
        --------
        >>> VolumeControlParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumeControlParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._cell_zonelet_type is not None:
            json_data["cellZoneletType"] = self._cell_zonelet_type
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "cell_zonelet_type :  %s" % (self._cell_zonelet_type)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def cell_zonelet_type(self) -> CellZoneletType:
        """Cell zonelet type is used to define the type of the associated volume.
        """
        return self._cell_zonelet_type

    @cell_zonelet_type.setter
    def cell_zonelet_type(self, value: CellZoneletType):
        self._cell_zonelet_type = value



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class ExtractFeatureParams(CoreObject):
    """Parameter to control feature edge extraction.
    """
    _default_params = {}

    def __initialize(
            self,
            replace: bool,
            feature_angle: float,
            separate_features: bool,
            separation_angle: float,
            disconnect_with_faces: bool,
            label_name: str):
        self._replace = replace
        self._feature_angle = feature_angle
        self._separate_features = separate_features
        self._separation_angle = separation_angle
        self._disconnect_with_faces = disconnect_with_faces
        self._label_name = label_name

    def __init__(
            self,
            model: CommunicationManager=None,
            replace: bool = None,
            feature_angle: float = None,
            separate_features: bool = None,
            separation_angle: float = None,
            disconnect_with_faces: bool = None,
            label_name: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ExtractFeatureParams.

        Parameters
        ----------
        model: Model
            Model to create a ExtractFeatureParams object with default parameters.
        replace: bool, optional
            Option to replace existing edge zonelets with new extracted edge zonelets.
        feature_angle: float, optional
            Angle used to capture face features to be  extracted as edges.
        separate_features: bool, optional
            Option to separate extracted features.
        separation_angle: float, optional
            Angle used to separate extracted features.
        disconnect_with_faces: bool, optional
            Option to disconnect edges from faces. If false, edges remain connected to faces by sharing nodes.
        label_name: str, optional
            Label name to be assigned to extracted features.
        json_data: dict, optional
            JSON dictionary to create a ExtractFeatureParams object with provided parameters.

        Examples
        --------
        >>> extract_feature_params = prime.ExtractFeatureParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["replace"] if "replace" in json_data else None,
                json_data["featureAngle"] if "featureAngle" in json_data else None,
                json_data["separateFeatures"] if "separateFeatures" in json_data else None,
                json_data["separationAngle"] if "separationAngle" in json_data else None,
                json_data["disconnectWithFaces"] if "disconnectWithFaces" in json_data else None,
                json_data["labelName"] if "labelName" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [replace, feature_angle, separate_features, separation_angle, disconnect_with_faces, label_name])
            if all_field_specified:
                self.__initialize(
                    replace,
                    feature_angle,
                    separate_features,
                    separation_angle,
                    disconnect_with_faces,
                    label_name)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ExtractFeatureParams")
                    json_data = param_json["ExtractFeatureParams"] if "ExtractFeatureParams" in param_json else {}
                    self.__initialize(
                        replace if replace is not None else ( ExtractFeatureParams._default_params["replace"] if "replace" in ExtractFeatureParams._default_params else (json_data["replace"] if "replace" in json_data else None)),
                        feature_angle if feature_angle is not None else ( ExtractFeatureParams._default_params["feature_angle"] if "feature_angle" in ExtractFeatureParams._default_params else (json_data["featureAngle"] if "featureAngle" in json_data else None)),
                        separate_features if separate_features is not None else ( ExtractFeatureParams._default_params["separate_features"] if "separate_features" in ExtractFeatureParams._default_params else (json_data["separateFeatures"] if "separateFeatures" in json_data else None)),
                        separation_angle if separation_angle is not None else ( ExtractFeatureParams._default_params["separation_angle"] if "separation_angle" in ExtractFeatureParams._default_params else (json_data["separationAngle"] if "separationAngle" in json_data else None)),
                        disconnect_with_faces if disconnect_with_faces is not None else ( ExtractFeatureParams._default_params["disconnect_with_faces"] if "disconnect_with_faces" in ExtractFeatureParams._default_params else (json_data["disconnectWithFaces"] if "disconnectWithFaces" in json_data else None)),
                        label_name if label_name is not None else ( ExtractFeatureParams._default_params["label_name"] if "label_name" in ExtractFeatureParams._default_params else (json_data["labelName"] if "labelName" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            replace: bool = None,
            feature_angle: float = None,
            separate_features: bool = None,
            separation_angle: float = None,
            disconnect_with_faces: bool = None,
            label_name: str = None):
        """Set the default values of ExtractFeatureParams.

        Parameters
        ----------
        replace: bool, optional
            Option to replace existing edge zonelets with new extracted edge zonelets.
        feature_angle: float, optional
            Angle used to capture face features to be  extracted as edges.
        separate_features: bool, optional
            Option to separate extracted features.
        separation_angle: float, optional
            Angle used to separate extracted features.
        disconnect_with_faces: bool, optional
            Option to disconnect edges from faces. If false, edges remain connected to faces by sharing nodes.
        label_name: str, optional
            Label name to be assigned to extracted features.
        """
        args = locals()
        [ExtractFeatureParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ExtractFeatureParams.

        Examples
        --------
        >>> ExtractFeatureParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ExtractFeatureParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._replace is not None:
            json_data["replace"] = self._replace
        if self._feature_angle is not None:
            json_data["featureAngle"] = self._feature_angle
        if self._separate_features is not None:
            json_data["separateFeatures"] = self._separate_features
        if self._separation_angle is not None:
            json_data["separationAngle"] = self._separation_angle
        if self._disconnect_with_faces is not None:
            json_data["disconnectWithFaces"] = self._disconnect_with_faces
        if self._label_name is not None:
            json_data["labelName"] = self._label_name
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "replace :  %s\nfeature_angle :  %s\nseparate_features :  %s\nseparation_angle :  %s\ndisconnect_with_faces :  %s\nlabel_name :  %s" % (self._replace, self._feature_angle, self._separate_features, self._separation_angle, self._disconnect_with_faces, self._label_name)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def replace(self) -> bool:
        """Option to replace existing edge zonelets with new extracted edge zonelets.
        """
        return self._replace

    @replace.setter
    def replace(self, value: bool):
        self._replace = value

    @property
    def feature_angle(self) -> float:
        """Angle used to capture face features to be  extracted as edges.
        """
        return self._feature_angle

    @feature_angle.setter
    def feature_angle(self, value: float):
        self._feature_angle = value

    @property
    def separate_features(self) -> bool:
        """Option to separate extracted features.
        """
        return self._separate_features

    @separate_features.setter
    def separate_features(self, value: bool):
        self._separate_features = value

    @property
    def separation_angle(self) -> float:
        """Angle used to separate extracted features.
        """
        return self._separation_angle

    @separation_angle.setter
    def separation_angle(self, value: float):
        self._separation_angle = value

    @property
    def disconnect_with_faces(self) -> bool:
        """Option to disconnect edges from faces. If false, edges remain connected to faces by sharing nodes.
        """
        return self._disconnect_with_faces

    @disconnect_with_faces.setter
    def disconnect_with_faces(self, value: bool):
        self._disconnect_with_faces = value

    @property
    def label_name(self) -> str:
        """Label name to be assigned to extracted features.
        """
        return self._label_name

    @label_name.setter
    def label_name(self, value: str):
        self._label_name = value

class ExtractFeatureResults(CoreObject):
    """Result of edge zonelet extraction by angle.
    """
    _default_params = {}

    def __initialize(
            self,
            processing_time: float,
            error_code: ErrorCode,
            new_edge_zonelets: Iterable[int]):
        self._processing_time = processing_time
        self._error_code = ErrorCode(error_code)
        self._new_edge_zonelets = new_edge_zonelets if isinstance(new_edge_zonelets, np.ndarray) else np.array(new_edge_zonelets, dtype=np.int32) if new_edge_zonelets is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            processing_time: float = None,
            error_code: ErrorCode = None,
            new_edge_zonelets: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ExtractFeatureResults.

        Parameters
        ----------
        model: Model
            Model to create a ExtractFeatureResults object with default parameters.
        processing_time: float, optional
            Time taken for edge extraction.
        error_code: ErrorCode, optional
            Error code returned by edge extraction function.
        new_edge_zonelets: Iterable[int], optional
            Ids of new edge zonelets extracted.
        json_data: dict, optional
            JSON dictionary to create a ExtractFeatureResults object with provided parameters.

        Examples
        --------
        >>> extract_feature_results = prime.ExtractFeatureResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["processingTime"] if "processingTime" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["newEdgeZonelets"] if "newEdgeZonelets" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [processing_time, error_code, new_edge_zonelets])
            if all_field_specified:
                self.__initialize(
                    processing_time,
                    error_code,
                    new_edge_zonelets)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ExtractFeatureResults")
                    json_data = param_json["ExtractFeatureResults"] if "ExtractFeatureResults" in param_json else {}
                    self.__initialize(
                        processing_time if processing_time is not None else ( ExtractFeatureResults._default_params["processing_time"] if "processing_time" in ExtractFeatureResults._default_params else (json_data["processingTime"] if "processingTime" in json_data else None)),
                        error_code if error_code is not None else ( ExtractFeatureResults._default_params["error_code"] if "error_code" in ExtractFeatureResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        new_edge_zonelets if new_edge_zonelets is not None else ( ExtractFeatureResults._default_params["new_edge_zonelets"] if "new_edge_zonelets" in ExtractFeatureResults._default_params else (json_data["newEdgeZonelets"] if "newEdgeZonelets" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            processing_time: float = None,
            error_code: ErrorCode = None,
            new_edge_zonelets: Iterable[int] = None):
        """Set the default values of ExtractFeatureResults.

        Parameters
        ----------
        processing_time: float, optional
            Time taken for edge extraction.
        error_code: ErrorCode, optional
            Error code returned by edge extraction function.
        new_edge_zonelets: Iterable[int], optional
            Ids of new edge zonelets extracted.
        """
        args = locals()
        [ExtractFeatureResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ExtractFeatureResults.

        Examples
        --------
        >>> ExtractFeatureResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ExtractFeatureResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._processing_time is not None:
            json_data["processingTime"] = self._processing_time
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._new_edge_zonelets is not None:
            json_data["newEdgeZonelets"] = self._new_edge_zonelets
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "processing_time :  %s\nerror_code :  %s\nnew_edge_zonelets :  %s" % (self._processing_time, self._error_code, self._new_edge_zonelets)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def processing_time(self) -> float:
        """Time taken for edge extraction.
        """
        return self._processing_time

    @processing_time.setter
    def processing_time(self, value: float):
        self._processing_time = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code returned by edge extraction function.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def new_edge_zonelets(self) -> Iterable[int]:
        """Ids of new edge zonelets extracted.
        """
        return self._new_edge_zonelets

    @new_edge_zonelets.setter
    def new_edge_zonelets(self, value: Iterable[int]):
        self._new_edge_zonelets = value

class ExtractedFeatureIds(CoreObject):
    """Contains ids of the features extracted.
    """
    _default_params = {}

    def __initialize(
            self,
            part_id: int,
            new_edge_zonelets: Iterable[int]):
        self._part_id = part_id
        self._new_edge_zonelets = new_edge_zonelets if isinstance(new_edge_zonelets, np.ndarray) else np.array(new_edge_zonelets, dtype=np.int32) if new_edge_zonelets is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            part_id: int = None,
            new_edge_zonelets: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the ExtractedFeatureIds.

        Parameters
        ----------
        model: Model
            Model to create a ExtractedFeatureIds object with default parameters.
        part_id: int, optional
            Id of the part from which edge zonelets are extracted.
        new_edge_zonelets: Iterable[int], optional
            Ids of new edge zonelets extracted.
        json_data: dict, optional
            JSON dictionary to create a ExtractedFeatureIds object with provided parameters.

        Examples
        --------
        >>> extracted_feature_ids = prime.ExtractedFeatureIds(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["partId"] if "partId" in json_data else None,
                json_data["newEdgeZonelets"] if "newEdgeZonelets" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [part_id, new_edge_zonelets])
            if all_field_specified:
                self.__initialize(
                    part_id,
                    new_edge_zonelets)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "ExtractedFeatureIds")
                    json_data = param_json["ExtractedFeatureIds"] if "ExtractedFeatureIds" in param_json else {}
                    self.__initialize(
                        part_id if part_id is not None else ( ExtractedFeatureIds._default_params["part_id"] if "part_id" in ExtractedFeatureIds._default_params else (json_data["partId"] if "partId" in json_data else None)),
                        new_edge_zonelets if new_edge_zonelets is not None else ( ExtractedFeatureIds._default_params["new_edge_zonelets"] if "new_edge_zonelets" in ExtractedFeatureIds._default_params else (json_data["newEdgeZonelets"] if "newEdgeZonelets" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            part_id: int = None,
            new_edge_zonelets: Iterable[int] = None):
        """Set the default values of ExtractedFeatureIds.

        Parameters
        ----------
        part_id: int, optional
            Id of the part from which edge zonelets are extracted.
        new_edge_zonelets: Iterable[int], optional
            Ids of new edge zonelets extracted.
        """
        args = locals()
        [ExtractedFeatureIds._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ExtractedFeatureIds.

        Examples
        --------
        >>> ExtractedFeatureIds.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ExtractedFeatureIds._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._part_id is not None:
            json_data["partId"] = self._part_id
        if self._new_edge_zonelets is not None:
            json_data["newEdgeZonelets"] = self._new_edge_zonelets
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "part_id :  %s\nnew_edge_zonelets :  %s" % (self._part_id, self._new_edge_zonelets)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def part_id(self) -> int:
        """Id of the part from which edge zonelets are extracted.
        """
        return self._part_id

    @part_id.setter
    def part_id(self, value: int):
        self._part_id = value

    @property
    def new_edge_zonelets(self) -> Iterable[int]:
        """Ids of new edge zonelets extracted.
        """
        return self._new_edge_zonelets

    @new_edge_zonelets.setter
    def new_edge_zonelets(self, value: Iterable[int]):
        self._new_edge_zonelets = value

class CreateIntersectionEdgeLoopsParams(CoreObject):
    """Parameters used to calculate edge loops created by intersection of two groups of face zonelets.
    """
    _default_params = {}

    def __initialize(
            self,
            label_name: str):
        self._label_name = label_name

    def __init__(
            self,
            model: CommunicationManager=None,
            label_name: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CreateIntersectionEdgeLoopsParams.

        Parameters
        ----------
        model: Model
            Model to create a CreateIntersectionEdgeLoopsParams object with default parameters.
        label_name: str, optional
            Label name to be assigned to extracted features.
        json_data: dict, optional
            JSON dictionary to create a CreateIntersectionEdgeLoopsParams object with provided parameters.

        Examples
        --------
        >>> create_intersection_edge_loops_params = prime.CreateIntersectionEdgeLoopsParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["labelName"] if "labelName" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [label_name])
            if all_field_specified:
                self.__initialize(
                    label_name)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CreateIntersectionEdgeLoopsParams")
                    json_data = param_json["CreateIntersectionEdgeLoopsParams"] if "CreateIntersectionEdgeLoopsParams" in param_json else {}
                    self.__initialize(
                        label_name if label_name is not None else ( CreateIntersectionEdgeLoopsParams._default_params["label_name"] if "label_name" in CreateIntersectionEdgeLoopsParams._default_params else (json_data["labelName"] if "labelName" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            label_name: str = None):
        """Set the default values of CreateIntersectionEdgeLoopsParams.

        Parameters
        ----------
        label_name: str, optional
            Label name to be assigned to extracted features.
        """
        args = locals()
        [CreateIntersectionEdgeLoopsParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CreateIntersectionEdgeLoopsParams.

        Examples
        --------
        >>> CreateIntersectionEdgeLoopsParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CreateIntersectionEdgeLoopsParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._label_name is not None:
            json_data["labelName"] = self._label_name
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "label_name :  %s" % (self._label_name)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def label_name(self) -> str:
        """Label name to be assigned to extracted features.
        """
        return self._label_name

    @label_name.setter
    def label_name(self, value: str):
        self._label_name = value

class CreateIntersectionEdgeLoopsResults(CoreObject):
    """Results for the edge loops created by intersection of two groups of face zonelets.
    """
    _default_params = {}

    def __initialize(
            self,
            processing_time: float,
            error_code: ErrorCode,
            extracted_ids: List[ExtractedFeatureIds]):
        self._processing_time = processing_time
        self._error_code = ErrorCode(error_code)
        self._extracted_ids = extracted_ids

    def __init__(
            self,
            model: CommunicationManager=None,
            processing_time: float = None,
            error_code: ErrorCode = None,
            extracted_ids: List[ExtractedFeatureIds] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CreateIntersectionEdgeLoopsResults.

        Parameters
        ----------
        model: Model
            Model to create a CreateIntersectionEdgeLoopsResults object with default parameters.
        processing_time: float, optional
            Time taken to extract edges formed by intersecting faces.
        error_code: ErrorCode, optional
            Error code returned by edge extraction function.
        extracted_ids: List[ExtractedFeatureIds], optional
            List of ExtractedFeatureIds that contains ids of extracted edges.
        json_data: dict, optional
            JSON dictionary to create a CreateIntersectionEdgeLoopsResults object with provided parameters.

        Examples
        --------
        >>> create_intersection_edge_loops_results = prime.CreateIntersectionEdgeLoopsResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["processingTime"] if "processingTime" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                [ExtractedFeatureIds(model = model, json_data = data) for data in json_data["extractedIds"]] if "extractedIds" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [processing_time, error_code, extracted_ids])
            if all_field_specified:
                self.__initialize(
                    processing_time,
                    error_code,
                    extracted_ids)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CreateIntersectionEdgeLoopsResults")
                    json_data = param_json["CreateIntersectionEdgeLoopsResults"] if "CreateIntersectionEdgeLoopsResults" in param_json else {}
                    self.__initialize(
                        processing_time if processing_time is not None else ( CreateIntersectionEdgeLoopsResults._default_params["processing_time"] if "processing_time" in CreateIntersectionEdgeLoopsResults._default_params else (json_data["processingTime"] if "processingTime" in json_data else None)),
                        error_code if error_code is not None else ( CreateIntersectionEdgeLoopsResults._default_params["error_code"] if "error_code" in CreateIntersectionEdgeLoopsResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        extracted_ids if extracted_ids is not None else ( CreateIntersectionEdgeLoopsResults._default_params["extracted_ids"] if "extracted_ids" in CreateIntersectionEdgeLoopsResults._default_params else [ExtractedFeatureIds(model = model, json_data = data) for data in (json_data["extractedIds"] if "extractedIds" in json_data else None)]))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            processing_time: float = None,
            error_code: ErrorCode = None,
            extracted_ids: List[ExtractedFeatureIds] = None):
        """Set the default values of CreateIntersectionEdgeLoopsResults.

        Parameters
        ----------
        processing_time: float, optional
            Time taken to extract edges formed by intersecting faces.
        error_code: ErrorCode, optional
            Error code returned by edge extraction function.
        extracted_ids: List[ExtractedFeatureIds], optional
            List of ExtractedFeatureIds that contains ids of extracted edges.
        """
        args = locals()
        [CreateIntersectionEdgeLoopsResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CreateIntersectionEdgeLoopsResults.

        Examples
        --------
        >>> CreateIntersectionEdgeLoopsResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CreateIntersectionEdgeLoopsResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._processing_time is not None:
            json_data["processingTime"] = self._processing_time
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._extracted_ids is not None:
            json_data["extractedIds"] = [data._jsonify() for data in self._extracted_ids]
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "processing_time :  %s\nerror_code :  %s\nextracted_ids :  %s" % (self._processing_time, self._error_code, '[' + ''.join('\n' + str(data) for data in self._extracted_ids) + ']')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def processing_time(self) -> float:
        """Time taken to extract edges formed by intersecting faces.
        """
        return self._processing_time

    @processing_time.setter
    def processing_time(self, value: float):
        self._processing_time = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code returned by edge extraction function.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def extracted_ids(self) -> List[ExtractedFeatureIds]:
        """List of ExtractedFeatureIds that contains ids of extracted edges.
        """
        return self._extracted_ids

    @extracted_ids.setter
    def extracted_ids(self, value: List[ExtractedFeatureIds]):
        self._extracted_ids = value



from ansys.meshing.prime.autogen.surfaceutilities import (
    SurfaceUtilities as _SurfaceUtilities,
)

# isort: split
from ansys.meshing.prime.autogen.surfaceutilitystructs import (
    AddThicknessParams as AddParams,
)
from ansys.meshing.prime.autogen.surfaceutilitystructs import (
    AddThicknessResults as AddResults,
)
from ansys.meshing.prime.autogen.surfaceutilitystructs import (
    CreateBOIParams as CreateBOIParams,
)
from ansys.meshing.prime.autogen.surfaceutilitystructs import (
    CreateBOIResults as CreateBOIResults,
)
from ansys.meshing.prime.autogen.surfaceutilitystructs import (
    CreateContactPatchParams as CPP,
)
from ansys.meshing.prime.autogen.surfaceutilitystructs import (
    CreateContactPatchResults as CPR,
)
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.params.primestructs import ErrorCode as ErrorCode
from ansys.meshing.prime.params.primestructs import Iterable as Iterable
from ansys.meshing.prime.params.primestructs import ScopeDefinition as SD


class SurfaceUtilities(_SurfaceUtilities):
    """Performs various general surface utilities algorithms. For example, add thickness."""

    def __init__(self, model: Model):
        """__init__(Model self, int id, int object_id, char* name)"""
        _SurfaceUtilities.__init__(self, model)
        self._model = model

    def add_thickness(self, zonelets: Iterable[int], params: AddParams) -> AddResults:
        """Adds thickness to the selected list of face zonelet ids.


        Parameters
        ----------
        zonelets : Iterable[int]
            List of input face zonelet ids.
        params : AddThicknessParams
            Parameters to control the add thickness operation.

        Returns
        -------
        AddThicknessResults
            Returns the AddThicknessResults.


        Examples
        --------
        >>> result = surf_utils.add_thickness(zonelets, params)

        """
        result = _SurfaceUtilities.add_thickness(self, zonelets, params)
        if result.error_code == ErrorCode.NOERROR:
            self._model._sync_up_model()
        return result

    def create_boi(self, scope: SD, params: CreateBOIParams) -> CreateBOIResults:
        """Creates BOI to the selected list of face zonelet ids.


        Parameters
        ----------
        scope : ScopeDefinition
            Scope of zonelets.
        params : CreateBOIParams
            Parameters to control the BOI creation operation.

        Returns
        -------
        CreateBOIResults
            Returns the BOIResults.


        Examples
        --------
        >>> result = surf_utils.create_surface_boi(zonelets, params)

        """
        result = _SurfaceUtilities.create_boi(self, scope, params)
        if result.error_code == ErrorCode.NOERROR:
            self._model._sync_up_model()
        return result

    def create_contact_patch(self, source_scope: SD, target_scope: SD, params: CPP) -> CPR:
        """Creates contact patches.


        Parameters
        ----------
        source_scope : ScopeDefinition
            Scope of source zonelets.
        target_scope : ScopeDefinition
            Scope of target zonelets which is to be offsetted for contact patch creation.
        params : CreateContactPatchParams
            Parameters to control the contact patch creation operation.

        Returns
        -------
        CreateContactPatchResults
            Returns the CreateContactPatchResults.


        Examples
        --------
        >>> result = surf_utils.create_contact_patch(zonelets, params)

        """
        result = _SurfaceUtilities.create_contact_patch(self, source_scope, target_scope, params)
        if result.error_code == ErrorCode.NOERROR:
            self._model._sync_up_model()
        return result



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class CellStatisticsParams(CoreObject):
    """Parameters used to calculate cell statistics.
    """
    _default_params = {}

    def __initialize(
            self,
            get_volume: bool):
        self._get_volume = get_volume

    def __init__(
            self,
            model: CommunicationManager=None,
            get_volume: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CellStatisticsParams.

        Parameters
        ----------
        model: Model
            Model to create a CellStatisticsParams object with default parameters.
        get_volume: bool, optional
            Provides option to compute and get cumulative cell volume.
        json_data: dict, optional
            JSON dictionary to create a CellStatisticsParams object with provided parameters.

        Examples
        --------
        >>> cell_statistics_params = prime.CellStatisticsParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["getVolume"] if "getVolume" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [get_volume])
            if all_field_specified:
                self.__initialize(
                    get_volume)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CellStatisticsParams")
                    json_data = param_json["CellStatisticsParams"] if "CellStatisticsParams" in param_json else {}
                    self.__initialize(
                        get_volume if get_volume is not None else ( CellStatisticsParams._default_params["get_volume"] if "get_volume" in CellStatisticsParams._default_params else (json_data["getVolume"] if "getVolume" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            get_volume: bool = None):
        """Set the default values of CellStatisticsParams.

        Parameters
        ----------
        get_volume: bool, optional
            Provides option to compute and get cumulative cell volume.
        """
        args = locals()
        [CellStatisticsParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CellStatisticsParams.

        Examples
        --------
        >>> CellStatisticsParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CellStatisticsParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._get_volume is not None:
            json_data["getVolume"] = self._get_volume
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "get_volume :  %s" % (self._get_volume)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def get_volume(self) -> bool:
        """Provides option to compute and get cumulative cell volume.
        """
        return self._get_volume

    @get_volume.setter
    def get_volume(self, value: bool):
        self._get_volume = value

class CellStatisticsResults(CoreObject):
    """Results of cell statistics information.
    """
    _default_params = {}

    def __initialize(
            self,
            volume: float,
            error_code: ErrorCode):
        self._volume = volume
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            volume: float = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the CellStatisticsResults.

        Parameters
        ----------
        model: Model
            Model to create a CellStatisticsResults object with default parameters.
        volume: float, optional
            Cumulative volume of all the cell elements of selected entities.
        error_code: ErrorCode, optional
            Error code associated with the cell statistics function.
        json_data: dict, optional
            JSON dictionary to create a CellStatisticsResults object with provided parameters.

        Examples
        --------
        >>> cell_statistics_results = prime.CellStatisticsResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["volume"] if "volume" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [volume, error_code])
            if all_field_specified:
                self.__initialize(
                    volume,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "CellStatisticsResults")
                    json_data = param_json["CellStatisticsResults"] if "CellStatisticsResults" in param_json else {}
                    self.__initialize(
                        volume if volume is not None else ( CellStatisticsResults._default_params["volume"] if "volume" in CellStatisticsResults._default_params else (json_data["volume"] if "volume" in json_data else None)),
                        error_code if error_code is not None else ( CellStatisticsResults._default_params["error_code"] if "error_code" in CellStatisticsResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            volume: float = None,
            error_code: ErrorCode = None):
        """Set the default values of CellStatisticsResults.

        Parameters
        ----------
        volume: float, optional
            Cumulative volume of all the cell elements of selected entities.
        error_code: ErrorCode, optional
            Error code associated with the cell statistics function.
        """
        args = locals()
        [CellStatisticsResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of CellStatisticsResults.

        Examples
        --------
        >>> CellStatisticsResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in CellStatisticsResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._volume is not None:
            json_data["volume"] = self._volume
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "volume :  %s\nerror_code :  %s" % (self._volume, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def volume(self) -> float:
        """Cumulative volume of all the cell elements of selected entities.
        """
        return self._volume

    @volume.setter
    def volume(self, value: float):
        self._volume = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the cell statistics function.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class FaceConnectivityResults(CoreObject):
    """Result of the face connectivity information.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            face_zonelet_ids: Iterable[int],
            topo_face_ids: Iterable[int],
            mesh_face_ids: Iterable[int],
            face_zone_ids: Iterable[int],
            face_zone_names: List[str],
            num_nodes_per_face_zonelet: Iterable[int],
            node_coords: Iterable[float],
            num_face_list_per_face_zonelet: Iterable[int],
            face_list: Iterable[int]):
        self._error_code = ErrorCode(error_code)
        self._face_zonelet_ids = face_zonelet_ids if isinstance(face_zonelet_ids, np.ndarray) else np.array(face_zonelet_ids, dtype=np.int32) if face_zonelet_ids is not None else None
        self._topo_face_ids = topo_face_ids if isinstance(topo_face_ids, np.ndarray) else np.array(topo_face_ids, dtype=np.int32) if topo_face_ids is not None else None
        self._mesh_face_ids = mesh_face_ids if isinstance(mesh_face_ids, np.ndarray) else np.array(mesh_face_ids, dtype=np.int32) if mesh_face_ids is not None else None
        self._face_zone_ids = face_zone_ids if isinstance(face_zone_ids, np.ndarray) else np.array(face_zone_ids, dtype=np.int32) if face_zone_ids is not None else None
        self._face_zone_names = face_zone_names
        self._num_nodes_per_face_zonelet = num_nodes_per_face_zonelet if isinstance(num_nodes_per_face_zonelet, np.ndarray) else np.array(num_nodes_per_face_zonelet, dtype=np.int32) if num_nodes_per_face_zonelet is not None else None
        self._node_coords = node_coords if isinstance(node_coords, np.ndarray) else np.array(node_coords, dtype=np.double) if node_coords is not None else None
        self._num_face_list_per_face_zonelet = num_face_list_per_face_zonelet if isinstance(num_face_list_per_face_zonelet, np.ndarray) else np.array(num_face_list_per_face_zonelet, dtype=np.int32) if num_face_list_per_face_zonelet is not None else None
        self._face_list = face_list if isinstance(face_list, np.ndarray) else np.array(face_list, dtype=np.int32) if face_list is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            face_zonelet_ids: Iterable[int] = None,
            topo_face_ids: Iterable[int] = None,
            mesh_face_ids: Iterable[int] = None,
            face_zone_ids: Iterable[int] = None,
            face_zone_names: List[str] = None,
            num_nodes_per_face_zonelet: Iterable[int] = None,
            node_coords: Iterable[float] = None,
            num_face_list_per_face_zonelet: Iterable[int] = None,
            face_list: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the FaceConnectivityResults.

        Parameters
        ----------
        model: Model
            Model to create a FaceConnectivityResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the get face connectivity operation.
        face_zonelet_ids: Iterable[int], optional
            Face zonelet ids for which connectivity data is available.
        topo_face_ids: Iterable[int], optional
            TopoFace ids corresponding to each face zonelet id for topology based mesh.
        mesh_face_ids: Iterable[int], optional
            Mesh face ids corresponding to each topoface.
        face_zone_ids: Iterable[int], optional
            Face zone id corresponding to each topoface or face zonelet.
        face_zone_names: List[str], optional
            Face zone name corresponding to each topoface or face zonelet.
        num_nodes_per_face_zonelet: Iterable[int], optional
            Number of nodes per face zonelet.
        node_coords: Iterable[float], optional
            Node coordinates describing faces of face zonelet.
        num_face_list_per_face_zonelet: Iterable[int], optional
            Number of face list per face zonelet.
        face_list: Iterable[int], optional
            Face list describing connectivity of node coordinates.
        json_data: dict, optional
            JSON dictionary to create a FaceConnectivityResults object with provided parameters.

        Examples
        --------
        >>> face_connectivity_results = prime.FaceConnectivityResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["faceZoneletIDs"] if "faceZoneletIDs" in json_data else None,
                json_data["topoFaceIDs"] if "topoFaceIDs" in json_data else None,
                json_data["meshFaceIDs"] if "meshFaceIDs" in json_data else None,
                json_data["faceZoneIDs"] if "faceZoneIDs" in json_data else None,
                json_data["faceZoneNames"] if "faceZoneNames" in json_data else None,
                json_data["numNodesPerFaceZonelet"] if "numNodesPerFaceZonelet" in json_data else None,
                json_data["nodeCoords"] if "nodeCoords" in json_data else None,
                json_data["numFaceListPerFaceZonelet"] if "numFaceListPerFaceZonelet" in json_data else None,
                json_data["faceList"] if "faceList" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, face_zonelet_ids, topo_face_ids, mesh_face_ids, face_zone_ids, face_zone_names, num_nodes_per_face_zonelet, node_coords, num_face_list_per_face_zonelet, face_list])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    face_zonelet_ids,
                    topo_face_ids,
                    mesh_face_ids,
                    face_zone_ids,
                    face_zone_names,
                    num_nodes_per_face_zonelet,
                    node_coords,
                    num_face_list_per_face_zonelet,
                    face_list)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "FaceConnectivityResults")
                    json_data = param_json["FaceConnectivityResults"] if "FaceConnectivityResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( FaceConnectivityResults._default_params["error_code"] if "error_code" in FaceConnectivityResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        face_zonelet_ids if face_zonelet_ids is not None else ( FaceConnectivityResults._default_params["face_zonelet_ids"] if "face_zonelet_ids" in FaceConnectivityResults._default_params else (json_data["faceZoneletIDs"] if "faceZoneletIDs" in json_data else None)),
                        topo_face_ids if topo_face_ids is not None else ( FaceConnectivityResults._default_params["topo_face_ids"] if "topo_face_ids" in FaceConnectivityResults._default_params else (json_data["topoFaceIDs"] if "topoFaceIDs" in json_data else None)),
                        mesh_face_ids if mesh_face_ids is not None else ( FaceConnectivityResults._default_params["mesh_face_ids"] if "mesh_face_ids" in FaceConnectivityResults._default_params else (json_data["meshFaceIDs"] if "meshFaceIDs" in json_data else None)),
                        face_zone_ids if face_zone_ids is not None else ( FaceConnectivityResults._default_params["face_zone_ids"] if "face_zone_ids" in FaceConnectivityResults._default_params else (json_data["faceZoneIDs"] if "faceZoneIDs" in json_data else None)),
                        face_zone_names if face_zone_names is not None else ( FaceConnectivityResults._default_params["face_zone_names"] if "face_zone_names" in FaceConnectivityResults._default_params else (json_data["faceZoneNames"] if "faceZoneNames" in json_data else None)),
                        num_nodes_per_face_zonelet if num_nodes_per_face_zonelet is not None else ( FaceConnectivityResults._default_params["num_nodes_per_face_zonelet"] if "num_nodes_per_face_zonelet" in FaceConnectivityResults._default_params else (json_data["numNodesPerFaceZonelet"] if "numNodesPerFaceZonelet" in json_data else None)),
                        node_coords if node_coords is not None else ( FaceConnectivityResults._default_params["node_coords"] if "node_coords" in FaceConnectivityResults._default_params else (json_data["nodeCoords"] if "nodeCoords" in json_data else None)),
                        num_face_list_per_face_zonelet if num_face_list_per_face_zonelet is not None else ( FaceConnectivityResults._default_params["num_face_list_per_face_zonelet"] if "num_face_list_per_face_zonelet" in FaceConnectivityResults._default_params else (json_data["numFaceListPerFaceZonelet"] if "numFaceListPerFaceZonelet" in json_data else None)),
                        face_list if face_list is not None else ( FaceConnectivityResults._default_params["face_list"] if "face_list" in FaceConnectivityResults._default_params else (json_data["faceList"] if "faceList" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            face_zonelet_ids: Iterable[int] = None,
            topo_face_ids: Iterable[int] = None,
            mesh_face_ids: Iterable[int] = None,
            face_zone_ids: Iterable[int] = None,
            face_zone_names: List[str] = None,
            num_nodes_per_face_zonelet: Iterable[int] = None,
            node_coords: Iterable[float] = None,
            num_face_list_per_face_zonelet: Iterable[int] = None,
            face_list: Iterable[int] = None):
        """Set the default values of FaceConnectivityResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the get face connectivity operation.
        face_zonelet_ids: Iterable[int], optional
            Face zonelet ids for which connectivity data is available.
        topo_face_ids: Iterable[int], optional
            TopoFace ids corresponding to each face zonelet id for topology based mesh.
        mesh_face_ids: Iterable[int], optional
            Mesh face ids corresponding to each topoface.
        face_zone_ids: Iterable[int], optional
            Face zone id corresponding to each topoface or face zonelet.
        face_zone_names: List[str], optional
            Face zone name corresponding to each topoface or face zonelet.
        num_nodes_per_face_zonelet: Iterable[int], optional
            Number of nodes per face zonelet.
        node_coords: Iterable[float], optional
            Node coordinates describing faces of face zonelet.
        num_face_list_per_face_zonelet: Iterable[int], optional
            Number of face list per face zonelet.
        face_list: Iterable[int], optional
            Face list describing connectivity of node coordinates.
        """
        args = locals()
        [FaceConnectivityResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of FaceConnectivityResults.

        Examples
        --------
        >>> FaceConnectivityResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in FaceConnectivityResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._face_zonelet_ids is not None:
            json_data["faceZoneletIDs"] = self._face_zonelet_ids
        if self._topo_face_ids is not None:
            json_data["topoFaceIDs"] = self._topo_face_ids
        if self._mesh_face_ids is not None:
            json_data["meshFaceIDs"] = self._mesh_face_ids
        if self._face_zone_ids is not None:
            json_data["faceZoneIDs"] = self._face_zone_ids
        if self._face_zone_names is not None:
            json_data["faceZoneNames"] = self._face_zone_names
        if self._num_nodes_per_face_zonelet is not None:
            json_data["numNodesPerFaceZonelet"] = self._num_nodes_per_face_zonelet
        if self._node_coords is not None:
            json_data["nodeCoords"] = self._node_coords
        if self._num_face_list_per_face_zonelet is not None:
            json_data["numFaceListPerFaceZonelet"] = self._num_face_list_per_face_zonelet
        if self._face_list is not None:
            json_data["faceList"] = self._face_list
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nface_zonelet_ids :  %s\ntopo_face_ids :  %s\nmesh_face_ids :  %s\nface_zone_ids :  %s\nface_zone_names :  %s\nnum_nodes_per_face_zonelet :  %s\nnode_coords :  %s\nnum_face_list_per_face_zonelet :  %s\nface_list :  %s" % (self._error_code, self._face_zonelet_ids, self._topo_face_ids, self._mesh_face_ids, self._face_zone_ids, self._face_zone_names, self._num_nodes_per_face_zonelet, self._node_coords, self._num_face_list_per_face_zonelet, self._face_list)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the get face connectivity operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def face_zonelet_ids(self) -> Iterable[int]:
        """Face zonelet ids for which connectivity data is available.
        """
        return self._face_zonelet_ids

    @face_zonelet_ids.setter
    def face_zonelet_ids(self, value: Iterable[int]):
        self._face_zonelet_ids = value

    @property
    def topo_face_ids(self) -> Iterable[int]:
        """TopoFace ids corresponding to each face zonelet id for topology based mesh.
        """
        return self._topo_face_ids

    @topo_face_ids.setter
    def topo_face_ids(self, value: Iterable[int]):
        self._topo_face_ids = value

    @property
    def mesh_face_ids(self) -> Iterable[int]:
        """Mesh face ids corresponding to each topoface.
        """
        return self._mesh_face_ids

    @mesh_face_ids.setter
    def mesh_face_ids(self, value: Iterable[int]):
        self._mesh_face_ids = value

    @property
    def face_zone_ids(self) -> Iterable[int]:
        """Face zone id corresponding to each topoface or face zonelet.
        """
        return self._face_zone_ids

    @face_zone_ids.setter
    def face_zone_ids(self, value: Iterable[int]):
        self._face_zone_ids = value

    @property
    def face_zone_names(self) -> List[str]:
        """Face zone name corresponding to each topoface or face zonelet.
        """
        return self._face_zone_names

    @face_zone_names.setter
    def face_zone_names(self, value: List[str]):
        self._face_zone_names = value

    @property
    def num_nodes_per_face_zonelet(self) -> Iterable[int]:
        """Number of nodes per face zonelet.
        """
        return self._num_nodes_per_face_zonelet

    @num_nodes_per_face_zonelet.setter
    def num_nodes_per_face_zonelet(self, value: Iterable[int]):
        self._num_nodes_per_face_zonelet = value

    @property
    def node_coords(self) -> Iterable[float]:
        """Node coordinates describing faces of face zonelet.
        """
        return self._node_coords

    @node_coords.setter
    def node_coords(self, value: Iterable[float]):
        self._node_coords = value

    @property
    def num_face_list_per_face_zonelet(self) -> Iterable[int]:
        """Number of face list per face zonelet.
        """
        return self._num_face_list_per_face_zonelet

    @num_face_list_per_face_zonelet.setter
    def num_face_list_per_face_zonelet(self, value: Iterable[int]):
        self._num_face_list_per_face_zonelet = value

    @property
    def face_list(self) -> Iterable[int]:
        """Face list describing connectivity of node coordinates.
        """
        return self._face_list

    @face_list.setter
    def face_list(self, value: Iterable[int]):
        self._face_list = value

class EdgeConnectivityResults(CoreObject):
    """Result of the edge connectivity information.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            edge_zonelet_ids: Iterable[int],
            topo_edge_ids: Iterable[int],
            mesh_edge_ids: Iterable[int],
            topo_edge_types: Iterable[int],
            num_nodes_per_edge_zonelet: Iterable[int],
            node_coords: Iterable[float],
            num_edge_list_per_edge_zonelet: Iterable[int],
            edge_list: Iterable[int],
            num_edges_per_edge_zonelet: Iterable[int]):
        self._error_code = ErrorCode(error_code)
        self._edge_zonelet_ids = edge_zonelet_ids if isinstance(edge_zonelet_ids, np.ndarray) else np.array(edge_zonelet_ids, dtype=np.int32) if edge_zonelet_ids is not None else None
        self._topo_edge_ids = topo_edge_ids if isinstance(topo_edge_ids, np.ndarray) else np.array(topo_edge_ids, dtype=np.int32) if topo_edge_ids is not None else None
        self._mesh_edge_ids = mesh_edge_ids if isinstance(mesh_edge_ids, np.ndarray) else np.array(mesh_edge_ids, dtype=np.int32) if mesh_edge_ids is not None else None
        self._topo_edge_types = topo_edge_types if isinstance(topo_edge_types, np.ndarray) else np.array(topo_edge_types, dtype=np.int32) if topo_edge_types is not None else None
        self._num_nodes_per_edge_zonelet = num_nodes_per_edge_zonelet if isinstance(num_nodes_per_edge_zonelet, np.ndarray) else np.array(num_nodes_per_edge_zonelet, dtype=np.int32) if num_nodes_per_edge_zonelet is not None else None
        self._node_coords = node_coords if isinstance(node_coords, np.ndarray) else np.array(node_coords, dtype=np.double) if node_coords is not None else None
        self._num_edge_list_per_edge_zonelet = num_edge_list_per_edge_zonelet if isinstance(num_edge_list_per_edge_zonelet, np.ndarray) else np.array(num_edge_list_per_edge_zonelet, dtype=np.int32) if num_edge_list_per_edge_zonelet is not None else None
        self._edge_list = edge_list if isinstance(edge_list, np.ndarray) else np.array(edge_list, dtype=np.int32) if edge_list is not None else None
        self._num_edges_per_edge_zonelet = num_edges_per_edge_zonelet if isinstance(num_edges_per_edge_zonelet, np.ndarray) else np.array(num_edges_per_edge_zonelet, dtype=np.int32) if num_edges_per_edge_zonelet is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            edge_zonelet_ids: Iterable[int] = None,
            topo_edge_ids: Iterable[int] = None,
            mesh_edge_ids: Iterable[int] = None,
            topo_edge_types: Iterable[int] = None,
            num_nodes_per_edge_zonelet: Iterable[int] = None,
            node_coords: Iterable[float] = None,
            num_edge_list_per_edge_zonelet: Iterable[int] = None,
            edge_list: Iterable[int] = None,
            num_edges_per_edge_zonelet: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the EdgeConnectivityResults.

        Parameters
        ----------
        model: Model
            Model to create a EdgeConnectivityResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the get edge connectivity operation.
        edge_zonelet_ids: Iterable[int], optional
            Edge zonelet ids for which connectivity data is available.
        topo_edge_ids: Iterable[int], optional
            TopoEdge ids corresponding to each edge zonelet id for topology based mesh.
        mesh_edge_ids: Iterable[int], optional
            Mesh edge ids corresponding to each topoedge.
        topo_edge_types: Iterable[int], optional
            TopoEdge type corresponding to each topoedge.
        num_nodes_per_edge_zonelet: Iterable[int], optional
            Number of nodes per edge zonelet.
        node_coords: Iterable[float], optional
            Node coordinates describing edges of edge zonelet.
        num_edge_list_per_edge_zonelet: Iterable[int], optional
            Number of edge list per edge zonelet.
        edge_list: Iterable[int], optional
            Edge list describing connectivity of node coordinates.
        num_edges_per_edge_zonelet: Iterable[int], optional
            Number of edges per edge zonelet.
        json_data: dict, optional
            JSON dictionary to create a EdgeConnectivityResults object with provided parameters.

        Examples
        --------
        >>> edge_connectivity_results = prime.EdgeConnectivityResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["edgeZoneletIDs"] if "edgeZoneletIDs" in json_data else None,
                json_data["topoEdgeIDs"] if "topoEdgeIDs" in json_data else None,
                json_data["meshEdgeIDs"] if "meshEdgeIDs" in json_data else None,
                json_data["topoEdgeTypes"] if "topoEdgeTypes" in json_data else None,
                json_data["numNodesPerEdgeZonelet"] if "numNodesPerEdgeZonelet" in json_data else None,
                json_data["nodeCoords"] if "nodeCoords" in json_data else None,
                json_data["numEdgeListPerEdgeZonelet"] if "numEdgeListPerEdgeZonelet" in json_data else None,
                json_data["edgeList"] if "edgeList" in json_data else None,
                json_data["numEdgesPerEdgeZonelet"] if "numEdgesPerEdgeZonelet" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, edge_zonelet_ids, topo_edge_ids, mesh_edge_ids, topo_edge_types, num_nodes_per_edge_zonelet, node_coords, num_edge_list_per_edge_zonelet, edge_list, num_edges_per_edge_zonelet])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    edge_zonelet_ids,
                    topo_edge_ids,
                    mesh_edge_ids,
                    topo_edge_types,
                    num_nodes_per_edge_zonelet,
                    node_coords,
                    num_edge_list_per_edge_zonelet,
                    edge_list,
                    num_edges_per_edge_zonelet)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "EdgeConnectivityResults")
                    json_data = param_json["EdgeConnectivityResults"] if "EdgeConnectivityResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( EdgeConnectivityResults._default_params["error_code"] if "error_code" in EdgeConnectivityResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        edge_zonelet_ids if edge_zonelet_ids is not None else ( EdgeConnectivityResults._default_params["edge_zonelet_ids"] if "edge_zonelet_ids" in EdgeConnectivityResults._default_params else (json_data["edgeZoneletIDs"] if "edgeZoneletIDs" in json_data else None)),
                        topo_edge_ids if topo_edge_ids is not None else ( EdgeConnectivityResults._default_params["topo_edge_ids"] if "topo_edge_ids" in EdgeConnectivityResults._default_params else (json_data["topoEdgeIDs"] if "topoEdgeIDs" in json_data else None)),
                        mesh_edge_ids if mesh_edge_ids is not None else ( EdgeConnectivityResults._default_params["mesh_edge_ids"] if "mesh_edge_ids" in EdgeConnectivityResults._default_params else (json_data["meshEdgeIDs"] if "meshEdgeIDs" in json_data else None)),
                        topo_edge_types if topo_edge_types is not None else ( EdgeConnectivityResults._default_params["topo_edge_types"] if "topo_edge_types" in EdgeConnectivityResults._default_params else (json_data["topoEdgeTypes"] if "topoEdgeTypes" in json_data else None)),
                        num_nodes_per_edge_zonelet if num_nodes_per_edge_zonelet is not None else ( EdgeConnectivityResults._default_params["num_nodes_per_edge_zonelet"] if "num_nodes_per_edge_zonelet" in EdgeConnectivityResults._default_params else (json_data["numNodesPerEdgeZonelet"] if "numNodesPerEdgeZonelet" in json_data else None)),
                        node_coords if node_coords is not None else ( EdgeConnectivityResults._default_params["node_coords"] if "node_coords" in EdgeConnectivityResults._default_params else (json_data["nodeCoords"] if "nodeCoords" in json_data else None)),
                        num_edge_list_per_edge_zonelet if num_edge_list_per_edge_zonelet is not None else ( EdgeConnectivityResults._default_params["num_edge_list_per_edge_zonelet"] if "num_edge_list_per_edge_zonelet" in EdgeConnectivityResults._default_params else (json_data["numEdgeListPerEdgeZonelet"] if "numEdgeListPerEdgeZonelet" in json_data else None)),
                        edge_list if edge_list is not None else ( EdgeConnectivityResults._default_params["edge_list"] if "edge_list" in EdgeConnectivityResults._default_params else (json_data["edgeList"] if "edgeList" in json_data else None)),
                        num_edges_per_edge_zonelet if num_edges_per_edge_zonelet is not None else ( EdgeConnectivityResults._default_params["num_edges_per_edge_zonelet"] if "num_edges_per_edge_zonelet" in EdgeConnectivityResults._default_params else (json_data["numEdgesPerEdgeZonelet"] if "numEdgesPerEdgeZonelet" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            edge_zonelet_ids: Iterable[int] = None,
            topo_edge_ids: Iterable[int] = None,
            mesh_edge_ids: Iterable[int] = None,
            topo_edge_types: Iterable[int] = None,
            num_nodes_per_edge_zonelet: Iterable[int] = None,
            node_coords: Iterable[float] = None,
            num_edge_list_per_edge_zonelet: Iterable[int] = None,
            edge_list: Iterable[int] = None,
            num_edges_per_edge_zonelet: Iterable[int] = None):
        """Set the default values of EdgeConnectivityResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the get edge connectivity operation.
        edge_zonelet_ids: Iterable[int], optional
            Edge zonelet ids for which connectivity data is available.
        topo_edge_ids: Iterable[int], optional
            TopoEdge ids corresponding to each edge zonelet id for topology based mesh.
        mesh_edge_ids: Iterable[int], optional
            Mesh edge ids corresponding to each topoedge.
        topo_edge_types: Iterable[int], optional
            TopoEdge type corresponding to each topoedge.
        num_nodes_per_edge_zonelet: Iterable[int], optional
            Number of nodes per edge zonelet.
        node_coords: Iterable[float], optional
            Node coordinates describing edges of edge zonelet.
        num_edge_list_per_edge_zonelet: Iterable[int], optional
            Number of edge list per edge zonelet.
        edge_list: Iterable[int], optional
            Edge list describing connectivity of node coordinates.
        num_edges_per_edge_zonelet: Iterable[int], optional
            Number of edges per edge zonelet.
        """
        args = locals()
        [EdgeConnectivityResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of EdgeConnectivityResults.

        Examples
        --------
        >>> EdgeConnectivityResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in EdgeConnectivityResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._edge_zonelet_ids is not None:
            json_data["edgeZoneletIDs"] = self._edge_zonelet_ids
        if self._topo_edge_ids is not None:
            json_data["topoEdgeIDs"] = self._topo_edge_ids
        if self._mesh_edge_ids is not None:
            json_data["meshEdgeIDs"] = self._mesh_edge_ids
        if self._topo_edge_types is not None:
            json_data["topoEdgeTypes"] = self._topo_edge_types
        if self._num_nodes_per_edge_zonelet is not None:
            json_data["numNodesPerEdgeZonelet"] = self._num_nodes_per_edge_zonelet
        if self._node_coords is not None:
            json_data["nodeCoords"] = self._node_coords
        if self._num_edge_list_per_edge_zonelet is not None:
            json_data["numEdgeListPerEdgeZonelet"] = self._num_edge_list_per_edge_zonelet
        if self._edge_list is not None:
            json_data["edgeList"] = self._edge_list
        if self._num_edges_per_edge_zonelet is not None:
            json_data["numEdgesPerEdgeZonelet"] = self._num_edges_per_edge_zonelet
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nedge_zonelet_ids :  %s\ntopo_edge_ids :  %s\nmesh_edge_ids :  %s\ntopo_edge_types :  %s\nnum_nodes_per_edge_zonelet :  %s\nnode_coords :  %s\nnum_edge_list_per_edge_zonelet :  %s\nedge_list :  %s\nnum_edges_per_edge_zonelet :  %s" % (self._error_code, self._edge_zonelet_ids, self._topo_edge_ids, self._mesh_edge_ids, self._topo_edge_types, self._num_nodes_per_edge_zonelet, self._node_coords, self._num_edge_list_per_edge_zonelet, self._edge_list, self._num_edges_per_edge_zonelet)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the get edge connectivity operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def edge_zonelet_ids(self) -> Iterable[int]:
        """Edge zonelet ids for which connectivity data is available.
        """
        return self._edge_zonelet_ids

    @edge_zonelet_ids.setter
    def edge_zonelet_ids(self, value: Iterable[int]):
        self._edge_zonelet_ids = value

    @property
    def topo_edge_ids(self) -> Iterable[int]:
        """TopoEdge ids corresponding to each edge zonelet id for topology based mesh.
        """
        return self._topo_edge_ids

    @topo_edge_ids.setter
    def topo_edge_ids(self, value: Iterable[int]):
        self._topo_edge_ids = value

    @property
    def mesh_edge_ids(self) -> Iterable[int]:
        """Mesh edge ids corresponding to each topoedge.
        """
        return self._mesh_edge_ids

    @mesh_edge_ids.setter
    def mesh_edge_ids(self, value: Iterable[int]):
        self._mesh_edge_ids = value

    @property
    def topo_edge_types(self) -> Iterable[int]:
        """TopoEdge type corresponding to each topoedge.
        """
        return self._topo_edge_types

    @topo_edge_types.setter
    def topo_edge_types(self, value: Iterable[int]):
        self._topo_edge_types = value

    @property
    def num_nodes_per_edge_zonelet(self) -> Iterable[int]:
        """Number of nodes per edge zonelet.
        """
        return self._num_nodes_per_edge_zonelet

    @num_nodes_per_edge_zonelet.setter
    def num_nodes_per_edge_zonelet(self, value: Iterable[int]):
        self._num_nodes_per_edge_zonelet = value

    @property
    def node_coords(self) -> Iterable[float]:
        """Node coordinates describing edges of edge zonelet.
        """
        return self._node_coords

    @node_coords.setter
    def node_coords(self, value: Iterable[float]):
        self._node_coords = value

    @property
    def num_edge_list_per_edge_zonelet(self) -> Iterable[int]:
        """Number of edge list per edge zonelet.
        """
        return self._num_edge_list_per_edge_zonelet

    @num_edge_list_per_edge_zonelet.setter
    def num_edge_list_per_edge_zonelet(self, value: Iterable[int]):
        self._num_edge_list_per_edge_zonelet = value

    @property
    def edge_list(self) -> Iterable[int]:
        """Edge list describing connectivity of node coordinates.
        """
        return self._edge_list

    @edge_list.setter
    def edge_list(self, value: Iterable[int]):
        self._edge_list = value

    @property
    def num_edges_per_edge_zonelet(self) -> Iterable[int]:
        """Number of edges per edge zonelet.
        """
        return self._num_edges_per_edge_zonelet

    @num_edges_per_edge_zonelet.setter
    def num_edges_per_edge_zonelet(self, value: Iterable[int]):
        self._num_edges_per_edge_zonelet = value

class FaceAndEdgeConnectivityResults(CoreObject):
    """Result of the face and edge connectivity information.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            part_ids: Iterable[int],
            face_connectivity_result_per_part: List[FaceConnectivityResults],
            edge_connectivity_result_per_part: List[EdgeConnectivityResults]):
        self._error_code = ErrorCode(error_code)
        self._part_ids = part_ids if isinstance(part_ids, np.ndarray) else np.array(part_ids, dtype=np.int32) if part_ids is not None else None
        self._face_connectivity_result_per_part = face_connectivity_result_per_part
        self._edge_connectivity_result_per_part = edge_connectivity_result_per_part

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            part_ids: Iterable[int] = None,
            face_connectivity_result_per_part: List[FaceConnectivityResults] = None,
            edge_connectivity_result_per_part: List[EdgeConnectivityResults] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the FaceAndEdgeConnectivityResults.

        Parameters
        ----------
        model: Model
            Model to create a FaceAndEdgeConnectivityResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the get face and edge connectivity operation.
        part_ids: Iterable[int], optional
            Part ids for which face and edge connectivity data is available.
        face_connectivity_result_per_part: List[FaceConnectivityResults], optional
            Face connectivity result per part.
        edge_connectivity_result_per_part: List[EdgeConnectivityResults], optional
            Edge connectivity result per part.
        json_data: dict, optional
            JSON dictionary to create a FaceAndEdgeConnectivityResults object with provided parameters.

        Examples
        --------
        >>> face_and_edge_connectivity_results = prime.FaceAndEdgeConnectivityResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["partIDs"] if "partIDs" in json_data else None,
                [FaceConnectivityResults(model = model, json_data = data) for data in json_data["faceConnectivityResultPerPart"]] if "faceConnectivityResultPerPart" in json_data else None,
                [EdgeConnectivityResults(model = model, json_data = data) for data in json_data["edgeConnectivityResultPerPart"]] if "edgeConnectivityResultPerPart" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, part_ids, face_connectivity_result_per_part, edge_connectivity_result_per_part])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    part_ids,
                    face_connectivity_result_per_part,
                    edge_connectivity_result_per_part)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "FaceAndEdgeConnectivityResults")
                    json_data = param_json["FaceAndEdgeConnectivityResults"] if "FaceAndEdgeConnectivityResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( FaceAndEdgeConnectivityResults._default_params["error_code"] if "error_code" in FaceAndEdgeConnectivityResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        part_ids if part_ids is not None else ( FaceAndEdgeConnectivityResults._default_params["part_ids"] if "part_ids" in FaceAndEdgeConnectivityResults._default_params else (json_data["partIDs"] if "partIDs" in json_data else None)),
                        face_connectivity_result_per_part if face_connectivity_result_per_part is not None else ( FaceAndEdgeConnectivityResults._default_params["face_connectivity_result_per_part"] if "face_connectivity_result_per_part" in FaceAndEdgeConnectivityResults._default_params else [FaceConnectivityResults(model = model, json_data = data) for data in (json_data["faceConnectivityResultPerPart"] if "faceConnectivityResultPerPart" in json_data else None)]),
                        edge_connectivity_result_per_part if edge_connectivity_result_per_part is not None else ( FaceAndEdgeConnectivityResults._default_params["edge_connectivity_result_per_part"] if "edge_connectivity_result_per_part" in FaceAndEdgeConnectivityResults._default_params else [EdgeConnectivityResults(model = model, json_data = data) for data in (json_data["edgeConnectivityResultPerPart"] if "edgeConnectivityResultPerPart" in json_data else None)]))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            part_ids: Iterable[int] = None,
            face_connectivity_result_per_part: List[FaceConnectivityResults] = None,
            edge_connectivity_result_per_part: List[EdgeConnectivityResults] = None):
        """Set the default values of FaceAndEdgeConnectivityResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the get face and edge connectivity operation.
        part_ids: Iterable[int], optional
            Part ids for which face and edge connectivity data is available.
        face_connectivity_result_per_part: List[FaceConnectivityResults], optional
            Face connectivity result per part.
        edge_connectivity_result_per_part: List[EdgeConnectivityResults], optional
            Edge connectivity result per part.
        """
        args = locals()
        [FaceAndEdgeConnectivityResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of FaceAndEdgeConnectivityResults.

        Examples
        --------
        >>> FaceAndEdgeConnectivityResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in FaceAndEdgeConnectivityResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._part_ids is not None:
            json_data["partIDs"] = self._part_ids
        if self._face_connectivity_result_per_part is not None:
            json_data["faceConnectivityResultPerPart"] = [data._jsonify() for data in self._face_connectivity_result_per_part]
        if self._edge_connectivity_result_per_part is not None:
            json_data["edgeConnectivityResultPerPart"] = [data._jsonify() for data in self._edge_connectivity_result_per_part]
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\npart_ids :  %s\nface_connectivity_result_per_part :  %s\nedge_connectivity_result_per_part :  %s" % (self._error_code, self._part_ids, '[' + ''.join('\n' + str(data) for data in self._face_connectivity_result_per_part) + ']', '[' + ''.join('\n' + str(data) for data in self._edge_connectivity_result_per_part) + ']')
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the get face and edge connectivity operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def part_ids(self) -> Iterable[int]:
        """Part ids for which face and edge connectivity data is available.
        """
        return self._part_ids

    @part_ids.setter
    def part_ids(self, value: Iterable[int]):
        self._part_ids = value

    @property
    def face_connectivity_result_per_part(self) -> List[FaceConnectivityResults]:
        """Face connectivity result per part.
        """
        return self._face_connectivity_result_per_part

    @face_connectivity_result_per_part.setter
    def face_connectivity_result_per_part(self, value: List[FaceConnectivityResults]):
        self._face_connectivity_result_per_part = value

    @property
    def edge_connectivity_result_per_part(self) -> List[EdgeConnectivityResults]:
        """Edge connectivity result per part.
        """
        return self._edge_connectivity_result_per_part

    @edge_connectivity_result_per_part.setter
    def edge_connectivity_result_per_part(self, value: List[EdgeConnectivityResults]):
        self._edge_connectivity_result_per_part = value

class FaceAndEdgeConnectivityParams(CoreObject):
    """Parameters to get face and edge connectivity information.
    """
    _default_params = {}

    def __initialize(
            self,
            reorder_face_zonelets_mid_nodes: bool,
            reorder_edge_zonelets_mid_nodes: bool):
        self._reorder_face_zonelets_mid_nodes = reorder_face_zonelets_mid_nodes
        self._reorder_edge_zonelets_mid_nodes = reorder_edge_zonelets_mid_nodes

    def __init__(
            self,
            model: CommunicationManager=None,
            reorder_face_zonelets_mid_nodes: bool = None,
            reorder_edge_zonelets_mid_nodes: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the FaceAndEdgeConnectivityParams.

        Parameters
        ----------
        model: Model
            Model to create a FaceAndEdgeConnectivityParams object with default parameters.
        reorder_face_zonelets_mid_nodes: bool, optional
            Option to reorder mid nodes for quadratic faces.
        reorder_edge_zonelets_mid_nodes: bool, optional
            Option to reorder mid nodes for quadratic edges.
        json_data: dict, optional
            JSON dictionary to create a FaceAndEdgeConnectivityParams object with provided parameters.

        Examples
        --------
        >>> face_and_edge_connectivity_params = prime.FaceAndEdgeConnectivityParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["reorderFaceZoneletsMidNodes"] if "reorderFaceZoneletsMidNodes" in json_data else None,
                json_data["reorderEdgeZoneletsMidNodes"] if "reorderEdgeZoneletsMidNodes" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [reorder_face_zonelets_mid_nodes, reorder_edge_zonelets_mid_nodes])
            if all_field_specified:
                self.__initialize(
                    reorder_face_zonelets_mid_nodes,
                    reorder_edge_zonelets_mid_nodes)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "FaceAndEdgeConnectivityParams")
                    json_data = param_json["FaceAndEdgeConnectivityParams"] if "FaceAndEdgeConnectivityParams" in param_json else {}
                    self.__initialize(
                        reorder_face_zonelets_mid_nodes if reorder_face_zonelets_mid_nodes is not None else ( FaceAndEdgeConnectivityParams._default_params["reorder_face_zonelets_mid_nodes"] if "reorder_face_zonelets_mid_nodes" in FaceAndEdgeConnectivityParams._default_params else (json_data["reorderFaceZoneletsMidNodes"] if "reorderFaceZoneletsMidNodes" in json_data else None)),
                        reorder_edge_zonelets_mid_nodes if reorder_edge_zonelets_mid_nodes is not None else ( FaceAndEdgeConnectivityParams._default_params["reorder_edge_zonelets_mid_nodes"] if "reorder_edge_zonelets_mid_nodes" in FaceAndEdgeConnectivityParams._default_params else (json_data["reorderEdgeZoneletsMidNodes"] if "reorderEdgeZoneletsMidNodes" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            reorder_face_zonelets_mid_nodes: bool = None,
            reorder_edge_zonelets_mid_nodes: bool = None):
        """Set the default values of FaceAndEdgeConnectivityParams.

        Parameters
        ----------
        reorder_face_zonelets_mid_nodes: bool, optional
            Option to reorder mid nodes for quadratic faces.
        reorder_edge_zonelets_mid_nodes: bool, optional
            Option to reorder mid nodes for quadratic edges.
        """
        args = locals()
        [FaceAndEdgeConnectivityParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of FaceAndEdgeConnectivityParams.

        Examples
        --------
        >>> FaceAndEdgeConnectivityParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in FaceAndEdgeConnectivityParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._reorder_face_zonelets_mid_nodes is not None:
            json_data["reorderFaceZoneletsMidNodes"] = self._reorder_face_zonelets_mid_nodes
        if self._reorder_edge_zonelets_mid_nodes is not None:
            json_data["reorderEdgeZoneletsMidNodes"] = self._reorder_edge_zonelets_mid_nodes
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "reorder_face_zonelets_mid_nodes :  %s\nreorder_edge_zonelets_mid_nodes :  %s" % (self._reorder_face_zonelets_mid_nodes, self._reorder_edge_zonelets_mid_nodes)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def reorder_face_zonelets_mid_nodes(self) -> bool:
        """Option to reorder mid nodes for quadratic faces.
        """
        return self._reorder_face_zonelets_mid_nodes

    @reorder_face_zonelets_mid_nodes.setter
    def reorder_face_zonelets_mid_nodes(self, value: bool):
        self._reorder_face_zonelets_mid_nodes = value

    @property
    def reorder_edge_zonelets_mid_nodes(self) -> bool:
        """Option to reorder mid nodes for quadratic edges.
        """
        return self._reorder_edge_zonelets_mid_nodes

    @reorder_edge_zonelets_mid_nodes.setter
    def reorder_edge_zonelets_mid_nodes(self, value: bool):
        self._reorder_edge_zonelets_mid_nodes = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

import logging
from ansys.meshing.prime.internals import utils

class Model(CoreObject, CommunicationManager):
    """Model is the nucleus of Prime. Model forms the base and contains all the information about Prime.

    You can access any information in Prime only through Model.
    Model allows you to query TopoData, ControlData, Parts, SizeFields and more.
    """

    def __init__(self, comm, id: int, object_id: int, name: str):
        """ Initialize Model """
        self._comm = comm
        self._logger = logging.getLogger("PyPrimeMesh")
        self._id = id
        self._object_id = object_id
        self._name = name

    def _print_logs_before_command(self, command, args):
        utils.print_logs_before_command(self._logger, command, args)

    def _print_logs_after_command(self, command, args = None):
        utils.print_logs_after_command(self._logger, command, args)

    def delete_parts(self, part_ids : Iterable[int]) -> DeleteResults:
        """ Delete the parts and its entities.


        Parameters
        ----------
        part_ids : Iterable[int]
            Ids of parts to be deleted.

        Returns
        -------
        DeleteResults
            Return the DeleteResults.


        Examples
        --------
        >>> results = model.delete_parts(part_ids)

        """
        args = {"part_ids" : part_ids}
        command_name = "PrimeMesh::Model/DeleteParts"
        self._print_logs_before_command("delete_parts", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("delete_parts", DeleteResults(model = self, json_data = result))
        return DeleteResults(model = self, json_data = result)

    def merge_parts(self, part_ids : Iterable[int], params : MergePartsParams) -> MergePartsResults:
        """ Merges given parts into one.


        Parameters
        ----------
        part_ids : Iterable[int]
            Ids of parts to be merged.
        params : MergePartsParams
            Parameters to merge parts.

        Returns
        -------
        MergePartsResults
            Returns the MergePartsResults.


        Examples
        --------
        >>> params = prime.MergePartsParams(model = model)
        >>> results = model.merge_parts(part_ids, params)

        """
        args = {"part_ids" : part_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Model/MergeParts"
        self._print_logs_before_command("merge_parts", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("merge_parts", MergePartsResults(model = self, json_data = result))
        return MergePartsResults(model = self, json_data = result)

    def set_global_sizing_params(self, params : GlobalSizingParams) -> SetSizingResults:
        """ Sets the global sizing parameters to initialize surfer parameters and various size control parameters.


        Parameters
        ----------
        params : GlobalSizingParams
            Global sizing parameters.

        Examples
        --------
        >>> model = client.model
        >>> model.set_global_sizing_params(
        >>>           prime.GlobalSizingParams(model=model,
        >>>           min = 0.1, max = 1.0, growth_rate = 1.2))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::Model/SetGlobalSizingParams"
        self._print_logs_before_command("set_global_sizing_params", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_global_sizing_params", SetSizingResults(model = self, json_data = result))
        return SetSizingResults(model = self, json_data = result)

    def get_active_volumetric_size_fields(self) -> Iterable[int]:
        """ Get the active sizefield ids.


        Returns
        -------
        Iterable[int]
            Return the list of active sizefield ids.


        Examples
        --------
        >>> model = client.model
        >>> active_size_field_ids = model.get_active_volumetric_size_fields()

        """
        args = {}
        command_name = "PrimeMesh::Model/GetActiveVolumetricSizeFields"
        self._print_logs_before_command("get_active_volumetric_size_fields", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_active_volumetric_size_fields")
        return result

    def get_volumetric_size_fields(self) -> Iterable[int]:
        """ Get the sizefield ids.


        Returns
        -------
        Iterable[int]
            Return the list of sizefield ids.


        Examples
        --------
        >>> model = client.model
        >>> size_field_ids = model.get_volumetric_size_fields()

        """
        args = {}
        command_name = "PrimeMesh::Model/GetVolumetricSizeFields"
        self._print_logs_before_command("get_volumetric_size_fields", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_volumetric_size_fields")
        return result

    def activate_volumetric_size_fields(self, size_field_ids : Iterable[int]):
        """ Activate the sizefields identified by the given sizefield ids.


        Parameters
        ----------
        size_field_ids : Iterable[int]
            List of sizefield ids.

        Examples
        --------
        >>> model = client.model
        >>> model.activate_volumetric_size_fields(size_field_ids)

        """
        args = {"size_field_ids" : size_field_ids}
        command_name = "PrimeMesh::Model/ActivateVolumetricSizeFields"
        self._print_logs_before_command("activate_volumetric_size_fields", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("activate_volumetric_size_fields")

    def deactivate_volumetric_size_fields(self, size_field_ids : Iterable[int]):
        """ Deactivate the sizefields identified by the given sizefield ids.


        Parameters
        ----------
        size_field_ids : Iterable[int]
            List of sizefield ids.

        Examples
        --------
        >>> model = client.model
        >>> model.deactivate_volumetric_size_fields(size_field_ids)

        """
        args = {"size_field_ids" : size_field_ids}
        command_name = "PrimeMesh::Model/DeactivateVolumetricSizeFields"
        self._print_logs_before_command("deactivate_volumetric_size_fields", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("deactivate_volumetric_size_fields")

    def delete_volumetric_size_fields(self, size_field_ids : Iterable[int]):
        """ Delete the sizefields identified by the given sizefield ids.


        Parameters
        ----------
        size_field_ids : Iterable[int]
            List of sizefield ids.

        Examples
        --------
        >>> model = client.model
        >>> model.delete_volumetric_size_fields(size_field_ids)

        """
        args = {"size_field_ids" : size_field_ids}
        command_name = "PrimeMesh::Model/DeleteVolumetricSizeFields"
        self._print_logs_before_command("delete_volumetric_size_fields", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("delete_volumetric_size_fields")

    def set_num_threads(self, num : int):
        """ Sets the number of threads for multithreaded operation.


        Parameters
        ----------
        num : int
            Number of threads.

        Examples
        --------
        >>> model = client.model
        >>> model.set_num_threads(4)

        """
        args = {"num" : num}
        command_name = "PrimeMesh::Model/SetNumThreads"
        self._print_logs_before_command("set_num_threads", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_num_threads")

    def get_num_threads(self) -> int:
        """ Get the number of threads enabled for multithreaded operation.


        Returns
        -------
        int
            Returns the number of threads.


        Examples
        --------
        >>> model = client.model
        >>> num_threads = model.get_num_threads()

        """
        args = {}
        command_name = "PrimeMesh::Model/GetNumThreads"
        self._print_logs_before_command("get_num_threads", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_num_threads")
        return result

    def start_distributed_meshing(self):
        """ Enables distributed meshing mode.


        Examples
        --------
        >>> model = prime.local_model()
        >>> model.start_distributed_meshing()

        """
        args = {}
        command_name = "PrimeMesh::Model/StartDistributedMeshing"
        self._print_logs_before_command("start_distributed_meshing", args)
        self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("start_distributed_meshing")

    def create_zone(self, suggested_name : str, type : ZoneType) -> CreateZoneResults:
        """ Creates zone for the given zone type.


        Parameters
        ----------
        suggested_name : str
            Suggested name for the zone to be created.
        type : ZoneType
            Type of the zone to be created.

        Returns
        -------
        CreateZoneResults
            Returns the CreateZoneResults.


        Examples
        --------
        >>> model = prime.local_model()
        >>> results = model.create_zone("wall", prime.ZoneType.FACE)

        """
        args = {"suggested_name" : suggested_name,
        "type" : type}
        command_name = "PrimeMesh::Model/CreateZone"
        self._print_logs_before_command("create_zone", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("create_zone", CreateZoneResults(model = self, json_data = result))
        return CreateZoneResults(model = self, json_data = result)

    def delete_zone(self, zone_id : int) -> DeleteZoneResults:
        """ Deletes zone identified with the given id.


        Parameters
        ----------
        zone_id : int
            Id of the zone to be deleted.

        Returns
        -------
        DeleteZoneResults
            Returns the DeleteZoneResults.


        Examples
        --------
        >>> model = prime.local_model()
        >>> results = model.delete_zone(1)

        """
        args = {"zone_id" : zone_id}
        command_name = "PrimeMesh::Model/DeleteZone"
        self._print_logs_before_command("delete_zone", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("delete_zone", DeleteZoneResults(model = self, json_data = result))
        return DeleteZoneResults(model = self, json_data = result)

    def get_zone_by_name(self, zone_name : str) -> int:
        """ Gets the zone by name.


        Parameters
        ----------
        zone_name : str
            Name of the zone.

        Returns
        -------
        int
            Returns id of the zone.


        Examples
        --------
        >>> model = prime.local_model()
        >>> zone_id = model.get_zone_by_name("inlet")

        """
        args = {"zone_name" : zone_name}
        command_name = "PrimeMesh::Model/GetZoneByName"
        self._print_logs_before_command("get_zone_by_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_zone_by_name")
        return result

    def get_zone_name(self, id : int) -> str:
        """ Get the name of given zone.


        Parameters
        ----------
        id : int
            Id of zone.

        Returns
        -------
        str
            Return the name. Return empty name if the id is invalid.


        Examples
        --------
        >>> model = prime.local_model()
        >>> name = model.get_zone_name(id)

        """
        args = {"id" : id}
        command_name = "PrimeMesh::Model/GetZoneName"
        self._print_logs_before_command("get_zone_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("get_zone_name")
        return result

    def set_suggested_zone_name(self, id : int, name : str) -> SetNameResults:
        """ Sets the unique name for zone with given id based on the given suggested name.


        Parameters
        ----------
        id : int
            Id of the zone to set suggested name.
        name : str
            Suggested name for the zone.

        Returns
        -------
        SetNameResults
            Returns the SetNameResults. results.assignedName indicates the assigned name to the zone.


        Examples
        --------
        >>> model = prime.local_model()
        >>> results = model.set_suggested_zone_name(id = 5, name = "zone1")

        """
        args = {"id" : id,
        "name" : name}
        command_name = "PrimeMesh::Model/SetSuggestedZoneName"
        self._print_logs_before_command("set_suggested_zone_name", args)
        result = self._comm.serve(self, command_name, self._object_id, args=args)
        self._print_logs_after_command("set_suggested_zone_name", SetNameResults(model = self, json_data = result))
        return SetNameResults(model = self, json_data = result)

    @property
    def id(self):
        """ Get the id of Model."""
        return self._id

    @property
    def name(self):
        """ Get the name of Model."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class VolumeQualityResultsPart(CoreObject):
    """Result of volume quality of part.
    """
    _default_params = {}

    def __initialize(
            self,
            cell_quality_measure: CellQualityMeasure,
            measure_name: str,
            part_id: int,
            quality_limit: float,
            n_found: int,
            max_quality: float,
            min_quality: float):
        self._cell_quality_measure = CellQualityMeasure(cell_quality_measure)
        self._measure_name = measure_name
        self._part_id = part_id
        self._quality_limit = quality_limit
        self._n_found = n_found
        self._max_quality = max_quality
        self._min_quality = min_quality

    def __init__(
            self,
            model: CommunicationManager=None,
            cell_quality_measure: CellQualityMeasure = None,
            measure_name: str = None,
            part_id: int = None,
            quality_limit: float = None,
            n_found: int = None,
            max_quality: float = None,
            min_quality: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumeQualityResultsPart.

        Parameters
        ----------
        model: Model
            Model to create a VolumeQualityResultsPart object with default parameters.
        cell_quality_measure: CellQualityMeasure, optional
            Type of the cell quality measure.
        measure_name: str, optional
            Name of the cell quality measure.
        part_id: int, optional
            Id of the part for which quality is computed.
        quality_limit: float, optional
            Target quality limit used to find failures.
        n_found: int, optional
            Number of failed cells.
        max_quality: float, optional
            Maximum value of quality measure.
        min_quality: float, optional
            Minimum value of quality measure.
        json_data: dict, optional
            JSON dictionary to create a VolumeQualityResultsPart object with provided parameters.

        Examples
        --------
        >>> volume_quality_results_part = prime.VolumeQualityResultsPart(model = model)
        """
        if json_data:
            self.__initialize(
                CellQualityMeasure(json_data["cellQualityMeasure"] if "cellQualityMeasure" in json_data else None),
                json_data["measureName"] if "measureName" in json_data else None,
                json_data["partId"] if "partId" in json_data else None,
                json_data["qualityLimit"] if "qualityLimit" in json_data else None,
                json_data["nFound"] if "nFound" in json_data else None,
                json_data["maxQuality"] if "maxQuality" in json_data else None,
                json_data["minQuality"] if "minQuality" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [cell_quality_measure, measure_name, part_id, quality_limit, n_found, max_quality, min_quality])
            if all_field_specified:
                self.__initialize(
                    cell_quality_measure,
                    measure_name,
                    part_id,
                    quality_limit,
                    n_found,
                    max_quality,
                    min_quality)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumeQualityResultsPart")
                    json_data = param_json["VolumeQualityResultsPart"] if "VolumeQualityResultsPart" in param_json else {}
                    self.__initialize(
                        cell_quality_measure if cell_quality_measure is not None else ( VolumeQualityResultsPart._default_params["cell_quality_measure"] if "cell_quality_measure" in VolumeQualityResultsPart._default_params else CellQualityMeasure(json_data["cellQualityMeasure"] if "cellQualityMeasure" in json_data else None)),
                        measure_name if measure_name is not None else ( VolumeQualityResultsPart._default_params["measure_name"] if "measure_name" in VolumeQualityResultsPart._default_params else (json_data["measureName"] if "measureName" in json_data else None)),
                        part_id if part_id is not None else ( VolumeQualityResultsPart._default_params["part_id"] if "part_id" in VolumeQualityResultsPart._default_params else (json_data["partId"] if "partId" in json_data else None)),
                        quality_limit if quality_limit is not None else ( VolumeQualityResultsPart._default_params["quality_limit"] if "quality_limit" in VolumeQualityResultsPart._default_params else (json_data["qualityLimit"] if "qualityLimit" in json_data else None)),
                        n_found if n_found is not None else ( VolumeQualityResultsPart._default_params["n_found"] if "n_found" in VolumeQualityResultsPart._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        max_quality if max_quality is not None else ( VolumeQualityResultsPart._default_params["max_quality"] if "max_quality" in VolumeQualityResultsPart._default_params else (json_data["maxQuality"] if "maxQuality" in json_data else None)),
                        min_quality if min_quality is not None else ( VolumeQualityResultsPart._default_params["min_quality"] if "min_quality" in VolumeQualityResultsPart._default_params else (json_data["minQuality"] if "minQuality" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            cell_quality_measure: CellQualityMeasure = None,
            measure_name: str = None,
            part_id: int = None,
            quality_limit: float = None,
            n_found: int = None,
            max_quality: float = None,
            min_quality: float = None):
        """Set the default values of VolumeQualityResultsPart.

        Parameters
        ----------
        cell_quality_measure: CellQualityMeasure, optional
            Type of the cell quality measure.
        measure_name: str, optional
            Name of the cell quality measure.
        part_id: int, optional
            Id of the part for which quality is computed.
        quality_limit: float, optional
            Target quality limit used to find failures.
        n_found: int, optional
            Number of failed cells.
        max_quality: float, optional
            Maximum value of quality measure.
        min_quality: float, optional
            Minimum value of quality measure.
        """
        args = locals()
        [VolumeQualityResultsPart._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumeQualityResultsPart.

        Examples
        --------
        >>> VolumeQualityResultsPart.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumeQualityResultsPart._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._cell_quality_measure is not None:
            json_data["cellQualityMeasure"] = self._cell_quality_measure
        if self._measure_name is not None:
            json_data["measureName"] = self._measure_name
        if self._part_id is not None:
            json_data["partId"] = self._part_id
        if self._quality_limit is not None:
            json_data["qualityLimit"] = self._quality_limit
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._max_quality is not None:
            json_data["maxQuality"] = self._max_quality
        if self._min_quality is not None:
            json_data["minQuality"] = self._min_quality
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "cell_quality_measure :  %s\nmeasure_name :  %s\npart_id :  %s\nquality_limit :  %s\nn_found :  %s\nmax_quality :  %s\nmin_quality :  %s" % (self._cell_quality_measure, self._measure_name, self._part_id, self._quality_limit, self._n_found, self._max_quality, self._min_quality)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def cell_quality_measure(self) -> CellQualityMeasure:
        """Type of the cell quality measure.
        """
        return self._cell_quality_measure

    @cell_quality_measure.setter
    def cell_quality_measure(self, value: CellQualityMeasure):
        self._cell_quality_measure = value

    @property
    def measure_name(self) -> str:
        """Name of the cell quality measure.
        """
        return self._measure_name

    @measure_name.setter
    def measure_name(self, value: str):
        self._measure_name = value

    @property
    def part_id(self) -> int:
        """Id of the part for which quality is computed.
        """
        return self._part_id

    @part_id.setter
    def part_id(self, value: int):
        self._part_id = value

    @property
    def quality_limit(self) -> float:
        """Target quality limit used to find failures.
        """
        return self._quality_limit

    @quality_limit.setter
    def quality_limit(self, value: float):
        self._quality_limit = value

    @property
    def n_found(self) -> int:
        """Number of failed cells.
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def max_quality(self) -> float:
        """Maximum value of quality measure.
        """
        return self._max_quality

    @max_quality.setter
    def max_quality(self, value: float):
        self._max_quality = value

    @property
    def min_quality(self) -> float:
        """Minimum value of quality measure.
        """
        return self._min_quality

    @min_quality.setter
    def min_quality(self, value: float):
        self._min_quality = value

class VolumeQualitySummaryResults(CoreObject):
    """Results of volume quality summary.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            quality_results_part: List[VolumeQualityResultsPart],
            message: str):
        self._error_code = ErrorCode(error_code)
        self._quality_results_part = quality_results_part
        self._message = message

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            quality_results_part: List[VolumeQualityResultsPart] = None,
            message: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumeQualitySummaryResults.

        Parameters
        ----------
        model: Model
            Model to create a VolumeQualitySummaryResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the volume quality summary.
        quality_results_part: List[VolumeQualityResultsPart], optional
            Contains volume quality result per cell quality measure by parts specified in parameters.
        message: str, optional
            Volume quality summary text.
        json_data: dict, optional
            JSON dictionary to create a VolumeQualitySummaryResults object with provided parameters.

        Examples
        --------
        >>> volume_quality_summary_results = prime.VolumeQualitySummaryResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                [VolumeQualityResultsPart(model = model, json_data = data) for data in json_data["qualityResultsPart"]] if "qualityResultsPart" in json_data else None,
                json_data["message"] if "message" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, quality_results_part, message])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    quality_results_part,
                    message)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumeQualitySummaryResults")
                    json_data = param_json["VolumeQualitySummaryResults"] if "VolumeQualitySummaryResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( VolumeQualitySummaryResults._default_params["error_code"] if "error_code" in VolumeQualitySummaryResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        quality_results_part if quality_results_part is not None else ( VolumeQualitySummaryResults._default_params["quality_results_part"] if "quality_results_part" in VolumeQualitySummaryResults._default_params else [VolumeQualityResultsPart(model = model, json_data = data) for data in (json_data["qualityResultsPart"] if "qualityResultsPart" in json_data else None)]),
                        message if message is not None else ( VolumeQualitySummaryResults._default_params["message"] if "message" in VolumeQualitySummaryResults._default_params else (json_data["message"] if "message" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            quality_results_part: List[VolumeQualityResultsPart] = None,
            message: str = None):
        """Set the default values of VolumeQualitySummaryResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the volume quality summary.
        quality_results_part: List[VolumeQualityResultsPart], optional
            Contains volume quality result per cell quality measure by parts specified in parameters.
        message: str, optional
            Volume quality summary text.
        """
        args = locals()
        [VolumeQualitySummaryResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumeQualitySummaryResults.

        Examples
        --------
        >>> VolumeQualitySummaryResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumeQualitySummaryResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._quality_results_part is not None:
            json_data["qualityResultsPart"] = [data._jsonify() for data in self._quality_results_part]
        if self._message is not None:
            json_data["message"] = self._message
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nquality_results_part :  %s\nmessage :  %s" % (self._error_code, '[' + ''.join('\n' + str(data) for data in self._quality_results_part) + ']', self._message)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the volume quality summary.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def quality_results_part(self) -> List[VolumeQualityResultsPart]:
        """Contains volume quality result per cell quality measure by parts specified in parameters.
        """
        return self._quality_results_part

    @quality_results_part.setter
    def quality_results_part(self, value: List[VolumeQualityResultsPart]):
        self._quality_results_part = value

    @property
    def message(self) -> str:
        """Volume quality summary text.
        """
        return self._message

    @message.setter
    def message(self, value: str):
        self._message = value

class VolumeQualitySummaryParams(CoreObject):
    """Parameters to control volume quality summary results.
    """
    _default_params = {}

    def __initialize(
            self,
            cell_quality_measures: List[CellQualityMeasure],
            scope: ScopeDefinition,
            quality_limit: Iterable[float]):
        self._cell_quality_measures = cell_quality_measures
        self._scope = scope
        self._quality_limit = quality_limit if isinstance(quality_limit, np.ndarray) else np.array(quality_limit, dtype=np.double) if quality_limit is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            cell_quality_measures: List[CellQualityMeasure] = None,
            scope: ScopeDefinition = None,
            quality_limit: Iterable[float] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the VolumeQualitySummaryParams.

        Parameters
        ----------
        model: Model
            Model to create a VolumeQualitySummaryParams object with default parameters.
        cell_quality_measures: List[CellQualityMeasure], optional
            List of cell quality measures for volume quality diagnostics.
        scope: ScopeDefinition, optional
            Scope of the cell zonelets for volume quality diagnostics.
        quality_limit: Iterable[float], optional
            Quality limit per cell quality measure. If the quality limit is not specified, the default quality limit is used.
        json_data: dict, optional
            JSON dictionary to create a VolumeQualitySummaryParams object with provided parameters.

        Examples
        --------
        >>> volume_quality_summary_params = prime.VolumeQualitySummaryParams(model = model)
        """
        if json_data:
            self.__initialize(
                [CellQualityMeasure(data) for data in json_data["cellQualityMeasures"]] if "cellQualityMeasures" in json_data else None,
                ScopeDefinition(model = model, json_data = json_data["scope"] if "scope" in json_data else None),
                json_data["qualityLimit"] if "qualityLimit" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [cell_quality_measures, scope, quality_limit])
            if all_field_specified:
                self.__initialize(
                    cell_quality_measures,
                    scope,
                    quality_limit)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumeQualitySummaryParams")
                    json_data = param_json["VolumeQualitySummaryParams"] if "VolumeQualitySummaryParams" in param_json else {}
                    self.__initialize(
                        cell_quality_measures if cell_quality_measures is not None else ( VolumeQualitySummaryParams._default_params["cell_quality_measures"] if "cell_quality_measures" in VolumeQualitySummaryParams._default_params else [CellQualityMeasure(data) for data in (json_data["cellQualityMeasures"] if "cellQualityMeasures" in json_data else None)]),
                        scope if scope is not None else ( VolumeQualitySummaryParams._default_params["scope"] if "scope" in VolumeQualitySummaryParams._default_params else ScopeDefinition(model = model, json_data = (json_data["scope"] if "scope" in json_data else None))),
                        quality_limit if quality_limit is not None else ( VolumeQualitySummaryParams._default_params["quality_limit"] if "quality_limit" in VolumeQualitySummaryParams._default_params else (json_data["qualityLimit"] if "qualityLimit" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            cell_quality_measures: List[CellQualityMeasure] = None,
            scope: ScopeDefinition = None,
            quality_limit: Iterable[float] = None):
        """Set the default values of VolumeQualitySummaryParams.

        Parameters
        ----------
        cell_quality_measures: List[CellQualityMeasure], optional
            List of cell quality measures for volume quality diagnostics.
        scope: ScopeDefinition, optional
            Scope of the cell zonelets for volume quality diagnostics.
        quality_limit: Iterable[float], optional
            Quality limit per cell quality measure. If the quality limit is not specified, the default quality limit is used.
        """
        args = locals()
        [VolumeQualitySummaryParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of VolumeQualitySummaryParams.

        Examples
        --------
        >>> VolumeQualitySummaryParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumeQualitySummaryParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._cell_quality_measures is not None:
            json_data["cellQualityMeasures"] = [data for data in self._cell_quality_measures]
        if self._scope is not None:
            json_data["scope"] = self._scope._jsonify()
        if self._quality_limit is not None:
            json_data["qualityLimit"] = self._quality_limit
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "cell_quality_measures :  %s\nscope :  %s\nquality_limit :  %s" % ('[' + ''.join('\n' + str(data) for data in self._cell_quality_measures) + ']', '{ ' + str(self._scope) + ' }', self._quality_limit)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def cell_quality_measures(self) -> List[CellQualityMeasure]:
        """List of cell quality measures for volume quality diagnostics.
        """
        return self._cell_quality_measures

    @cell_quality_measures.setter
    def cell_quality_measures(self, value: List[CellQualityMeasure]):
        self._cell_quality_measures = value

    @property
    def scope(self) -> ScopeDefinition:
        """Scope of the cell zonelets for volume quality diagnostics.
        """
        return self._scope

    @scope.setter
    def scope(self, value: ScopeDefinition):
        self._scope = value

    @property
    def quality_limit(self) -> Iterable[float]:
        """Quality limit per cell quality measure. If the quality limit is not specified, the default quality limit is used.
        """
        return self._quality_limit

    @quality_limit.setter
    def quality_limit(self, value: Iterable[float]):
        self._quality_limit = value
